'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol ? "symbol" : typeof obj; };

var _cluster = require('cluster');

var _cluster2 = _interopRequireDefault(_cluster);

var _os = require('os');

var _os2 = _interopRequireDefault(_os);

var _processName = require('./process-name');

var _processName2 = _interopRequireDefault(_processName);

var _logFactory = require('./log-factory');

var _logFactory2 = _interopRequireDefault(_logFactory);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var calculateOptions = function calculateOptions(options) {
    return Object.assign({
        numberOfWorkers: process.env.EXPRESS_CLUSTER_NUMBER_OF_WORKERS || _os2.default.cpus().length,
        handleUncaughtException: process.env.EXPRESS_CLUSTER_HANDLE_UNCAUGHT_EXCEPTION !== 'false',
        workerRespawnDelay: process.env.EXPRESS_CLUSTER_WORKER_RESPAWN_DELAY || 1000,
        workerKillTimeout: process.env.EXPRESS_CLUSTER_WORKER_KILL_TIMEOUT || 30000,
        logLevel: process.env.EXPRESS_CLUSTER_LOG_LEVEL || 'info'
    }, options);
};

var clusterStability = function clusterStability(workerFunction) {
    var options = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];
    var masterFunction = arguments[2];

    if (typeof workerFunction !== 'function') {
        throw new Error('workerFunction must be supplied as the first argument.');
    }
    if ((typeof options === 'undefined' ? 'undefined' : _typeof(options)) !== 'object') {
        throw new Error('The second argument, if supplied, must be an options object or null.');
    } else if (options !== null && (typeof options === 'undefined' ? 'undefined' : _typeof(options)) === 'object' && typeof options.length !== 'undefined') {
        throw new Error('The second argument, if supplied, must be an options object or null.');
    }
    if (typeof masterFunction !== 'undefined' && typeof masterFunction !== 'function') {
        throw new Error('The third argument, if supplied, must be the masterFunction.');
    }

    var effectiveOptions = calculateOptions(options);
    effectiveOptions.log = (0, _logFactory2.default)(effectiveOptions.logLevel, effectiveOptions.logger);
    if (_cluster2.default.isMaster) {
        require('./cluster-master')(masterFunction, effectiveOptions);
    } else {
        require('./cluster-worker')(workerFunction, effectiveOptions);
    }
};
clusterStability.processName = _processName2.default;
clusterStability.log = (0, _logFactory2.default)(calculateOptions().logLevel);

exports.default = clusterStability;
module.exports = exports['default'];