'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _processName = require('./process-name');

var _processName2 = _interopRequireDefault(_processName);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var levels = {
    fatal: 60,
    error: 50,
    warn: 40,
    info: 30,
    debug: 20,
    trace: 10
};

var noop = Object.assign(function () {}, { enabled: false });

var atLeast = function atLeast(min, level, fn) {
    return levels[level] >= levels[min] ? Object.assign(fn, { enabled: true }) : noop;
};

var defaultLogger = function defaultLogger(level) {
    return function (message) {
        for (var _len = arguments.length, rest = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
            rest[_key - 1] = arguments[_key];
        }

        if (['fatal', 'error', 'warn'].indexOf(level.toLowerCase()) !== -1) {
            var _console;

            (_console = console).error.apply(_console, [level.toUpperCase() + ': ' + _processName2.default + ': ' + message].concat(rest));
        } else {
            var _console2;

            (_console2 = console).log.apply(_console2, [level.toUpperCase() + ': ' + _processName2.default + ': ' + message].concat(rest));
        }
    };
};

exports.default = function (logLevel) {
    var logger = arguments.length <= 1 || arguments[1] === undefined ? defaultLogger : arguments[1];

    var log = atLeast(logLevel, 'info', logger('info'));
    log.info = atLeast(logLevel, 'info', logger('info'));
    log.debug = atLeast(logLevel, 'debug', logger('debug'));
    log.trace = atLeast(logLevel, 'trace', logger('trace'));
    log.fatal = atLeast(logLevel, 'fatal', logger('fatal'));
    log.warn = atLeast(logLevel, 'warn', logger('warn'));
    log.error = atLeast(logLevel, 'error', logger('error'));

    return log;
};

module.exports = exports['default'];